/// An element annotated with @nullable claims <code>null</code> value is perfectly <em>valid</em>
/// to return (for methods), pass to (parameters) and hold (local variables and fields).
///
/// Apart from documentation purposes this annotation is intended to be used by static analysis tools
/// to validate against probable runtime errors and element contract violations.
const Object nullable = _Nullable();

class _Nullable {
  const _Nullable();
}

/// An element annotated with @notNull claims <code>null</code> value is <em>forbidden</em>
/// to return (for methods), pass to (parameters) and hold (local variables and fields).
///
/// Apart from documentation purposes this annotation is intended to be used by static analysis tools
/// to validate against probable runtime errors and element contract violations.
const Object nonNull = _NonNull();

class _NonNull {
  const _NonNull();
}

/// A method annotated with @tooGeneric is a indicator that the method is defined for a generic type `T` but only works for type `X` where `X extends T`.
///
/// The method will be moved to a more specific type when dart extension methods are implemented.
/// https://github.com/dart-lang/language/issues/41
///
/// Use `@TooGeneric(type: X)` to define which type should be used
@Deprecated("Use static extension methods")
const Object tooGeneric = TooGeneric();

/// A method annotated with @tooGeneric is a indicator that the method is defined for a generic type `T` but only works for type `X` where `X extends T`.
///
/// The method will be moved to a more specific type when dart extension methods are implemented.
/// https://github.com/dart-lang/language/issues/41
///
/// Use `@TooGeneric(type: X)` to define which type should be used
@Deprecated("Use static extension methods")
class TooGeneric {
  const TooGeneric({this.extensionForType});

  /// The type which would be a better fit
  final String extensionForType;
}

/// A method/class annotated with @experimental marks the method/class as experimental feature.
///
/// Experimental APIs can be changed or removed at any time.
const Object experimental = _Experimental();

class _Experimental {
  const _Experimental();
}
